/*
** Copyright (c) 2002 D. Richard Hipp
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** You should have received a copy of the GNU General Public
** License along with this library; if not, write to the
** Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA  02111-1307, USA.
**
** Author contact information:
**   drh@hwaci.com
**   http://www.hwaci.com/drh/
**
*******************************************************************************
**
** Routines shared by many pages
*/
#include "config.h"
#include "common.h"

/*
** Generate an HTML header common to all web pages.  zTitle is the
** title for the page.  azLinks is an array of URI/Name pairs that
** are used to generate quick-links on the title bar.
*/
void common_header(const char *zTitle, const char **azLink){
  int i;
  @ <html>
  @ <head>
  @ <title>%h(zTitle)</title>
  @ </head>
  @ <body bgcolor="white">
  @ <table width="100%%" cellpadding=2 border=0>
  @ <tr><td bgcolor="%s(BORDER_COLOR)">
  @ <table width="100%%" border=0 cellpadding=2 cellspacing=0>
  @ <tr bgcolor="%s(BG_COLOR)">
  @ <td valign="top" align="left">
  @ <big><b>%h(g.zName) - %h(zTitle)</b></big><br>
  if( !g.isAnon ){
    @ <small>Logged in as <i>%h(g.zUser)</i></small>
  }else{
    @ <small>Not logged in</small>
  }
  @ </td>
  @ <td valign="bottom" align="right">
  for(i=0; azLink && azLink[i] && azLink[i+1]; i+=2){
    @ [<a href="%s(azLink[i])"><nobr>%h(azLink[i+1])</nobr></a>]&nbsp;
  }
  if( i==0 ){
    @ &nbsp;
  }
  @ </td></tr></table>
  @ </td></tr></table>
}

/*
** Generate a common footer
*/
void common_footer(void){
  @ <p><small><small>
  @ <a href="about">CVSTrac version @VERSION@</a>
  @ </small></small></p>
  @ </body></html>
}

/*
** Generate an about screen
**
** WEBPAGE: /about
*/
void common_about(void){
  static const char *azLink[] = { 
     "index", "Home", 
     0
  };
  login_check_credentials();
  common_header("About This Server", azLink);
  @ <p>This website is implemented using CVSTrac version @VERSION@.</p>
  @
  @ <p>CVSTrac implements a low-ceremony bug tracking system for small
  @ to medium-sized projects using CVS for version control.  For additional
  @ information, use your favorite search engine to locate the CVSTrac
  @ homepage.</p> 
  @
  @ <p>&copy; 2002 by <a href="mailto:drh@hwaci.com">D. Richard Hipp</a>.
  @ The CVSTrac server is released under the terms of the
  @ <a href="http://www.gnu.org/copyleft/gpl.html">GNU Public License</a>.</p>
  common_footer(); 
}
