/*
** Copyright (c) 2002 D. Richard Hipp
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** You should have received a copy of the GNU General Public
** License along with this library; if not, write to the
** Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA  02111-1307, USA.
**
** Author contact information:
**   drh@hwaci.com
**   http://www.hwaci.com/drh/
**
*******************************************************************************
**
** Routines for handling user account
*/
#define _XOPEN_SOURCE
#include <unistd.h>
#include "config.h"
#include "user.h"
#include <pwd.h>
#include <sys/types.h>

/*
** WEBPAGE: /userlist
*/
void user_list(void){
  static const char *azLink[] = {
    "index", "Home", "useredit", "Add User", 0
  };
  char **azResult;
  int i;

  login_check_credentials();
  if( !g.okWrite ){
    cgi_redirect("index");
    return;
  }
  common_header("User List", azLink);
  @ <table cellspacing=0 cellpadding=0 border=0>
  @ <tr>
  @   <th align="right"><nobr>User ID</nobr></th>
  @   <th>&nbsp;&nbsp;&nbsp;Permissions&nbsp;&nbsp;&nbsp;</th>
  @   <th><nobr>In Real Life</nobr></th>
  @ </tr>
  azResult = db_query(
    "SELECT id, name, email, capabilities FROM user ORDER BY id");
  for(i=0; azResult[i]; i+= 4){
    @ <tr>
    @ <td align="right">
    if( g.okAdmin ){
      @ <a href="useredit?id=%t(azResult[i])">
    }
    @ <nobr>%h(azResult[i])</nobr>
    if( g.okAdmin ){
      @ </a>
    }
    @ </td>
    @ <td align="center">%s(azResult[i+3])</td>
    if( azResult[i+2] && azResult[i+2][0] ){
      char *zE = azResult[i+2];
      @ <td align="left"><nobr>%h(azResult[i+1])
      @    (<a href="mailto:%h(zE)">%h(zE)</a>)</nobr></td>
    } else {
      @ <td align="left"><nobr>%h(azResult[i+1])</nobr></td>
    }
    @ </tr>
  }
  @ </table>
  @ <p><hr>
  @ <b>Notes:</b>
  @ <ol>
  @ <li><p>The permission flags are as follows:</p>
  @ <p><dl compact>
  @ <dt>a</dt><dd>Admin: Create or delete users and ticket report formats</dd>
  @ <dt>i</dt><dd>Check-in: Add new code to the CVS repository</dd>
  @ <dt>n</dt><dd>New: Create new tickets</dd>
  @ <dt>o</dt><dd>Check-out: Read code out of the CVS repository</dd>
  @ <dt>r</dt><dd>Read: View tickets and change histories</dd>
  @ <dt>s</dt><dd>Setup: Change the CVS repository</dd>
  @ <dt>w</dt><dd>Write: Edit tickets</dd>
  @ </dl>
  @ </p></li>
  @
  @ <li><p>
  @ If a user named "<b>anonymous</b>" exists, then anyone can access
  @ the server without having to log in.  The permissions on the
  @ anonymous user determine the access rights for anyone who is not
  @ logged in.
  @ </p></li>
  @
  @ <li><p>
  @ You must be using CVS version 1.11 or later in order to give users
  @ read-only access to the repository.
  @ With earlier versions of CVS, all users with check-out
  @ privileges also automatically get check-in privileges.
  @ </p></li>
  @
  @ <li><p>
  @ Changing a users ID or password rewrites the <b>CVSROOT/passwd</b> and
  @ <b>CVSROOT/writers</b> files in the CVS repository, if those files have
  @ write permission turned on.
  @ </p></li>
  @ </ol>
  common_footer();
}

/*
** WEBPAGE: /useredit
*/
void user_edit(void){
  static const char *azLink[] = {
    "index", "Home", "userlist", "User List", 0
  };
  char **azResult;
  const char *zId, *zName, *zEMail, *zCap, *zPW;
  char *oaa, *oas, *oar, *oaw, *oan, *oai, *oao;
  int doWrite;
  int higherUser = 0;

  login_check_credentials();
  if( !g.okAdmin ){ cgi_redirect("index"); return; }
  zId = P("id");
  if( zId && !g.okSetup ){
    char *zOldCaps;
    zOldCaps = db_short_query(
       "SELECT capabilities FROM user WHERE id='%q'",zId);
    higherUser = zOldCaps && strchr(zOldCaps,'s');
  }
  doWrite = zId && cgi_all("nm","em","pw") && !higherUser;
  if( doWrite ){
    char zCap[10];
    int i = 0;
    int aa = P("aa")!=0;
    int ai = P("ai")!=0;
    int an = P("an")!=0;
    int ao = P("ao")!=0;
    int ar = P("ar")!=0;
    int as = g.okSetup && P("as")!=0;
    int aw = P("aw")!=0;
    if( as ) aa = 1;
    if( aa ) ai = aw = 1;
    if( aw ) an = ar = 1;
    if( ai ) ao = 1;
    if( aa ){ zCap[i++] = 'a'; }
    if( ai ){ zCap[i++] = 'i'; }
    if( an ){ zCap[i++] = 'n'; }
    if( ao ){ zCap[i++] = 'o'; }
    if( ar ){ zCap[i++] = 'r'; }
    if( as ){ zCap[i++] = 's'; }
    if( aw ){ zCap[i++] = 'w'; }
    zCap[i] = 0;
    db_execute("DELETE FROM user WHERE id='%q'", P("id"));
    if( !P("delete") ){
      db_execute(
         "INSERT INTO user(id,name,email,passwd,capabilities) "
         "VALUES('%q','%q','%q','%q','%s')",
         zId, P("nm"), P("em"), P("pw"), zCap
      );
    }
    user_cvs_config();
    cgi_redirect("userlist");
    return;
  }
  zName = "";
  zEMail = "";
  zCap = "";
  zPW = "";
  oaa = oai = oan = oao = oar = oas = oaw = "";
  if( zId ){
    azResult = db_query(
      "SELECT name, email, capabilities, passwd FROM user WHERE id='%q'", zId
    );
    if( azResult && azResult[0] ){
      zName = azResult[0];
      zEMail = azResult[1];
      zCap = azResult[2];
      zPW = azResult[3];
      if( strchr(zCap, 'a') ) oaa = " checked";
      if( strchr(zCap, 'i') ) oai = " checked";
      if( strchr(zCap, 'n') ) oan = " checked";
      if( strchr(zCap, 'o') ) oao = " checked";
      if( strchr(zCap, 'r') ) oar = " checked";
      if( strchr(zCap, 's') ) oas = " checked";
      if( strchr(zCap, 'w') ) oaw = " checked";
    }else{
      zId = 0;
    }
  }
  if( zId ){
    common_header("Edit User", azLink);
  }else{
    common_header("Add New User", azLink);
  }
  if( higherUser ){ zPW = ""; }  
  @ <form action="%s(g.zPath)" method="POST">
  @ <table align="left" hspace=10 vspace=10>
  @ <tr>
  @   <td align="right"><nobr>User ID:</nobr></td>
  if( zId ){
    @   <td>%h(zId) <input type="hidden" name="id" value="%h(zId)"></td>
  }else{
    @   <td><input type="text" name="id" size=10></td>
  }
  @ </tr>
  @ <tr>
  @   <td align="right"><nobr>Full Name:</nobr></td>
  @   <td><input type="text" name="nm" value="%h(zName)"></td>
  @ </tr>
  @ <tr>
  @   <td align="right"><nobr>E-Mail:</nobr></td>
  @   <td><input type="text" name="em" value="%h(zEMail)"></td>
  @ </tr>
  @ <tr>
  @   <td align="right" valign="top">Capabilities:</td>
  @   <td>
  @     <input type="checkbox" name="aa"%s(oaa)>Admin</input><br>
  @     <input type="checkbox" name="ai"%s(oai)>Check-In</input><br>
  @     <input type="checkbox" name="an"%s(oan)>New Tkt</input><br>
  @     <input type="checkbox" name="ao"%s(oao)>Check-Out</input><br>
  @     <input type="checkbox" name="ar"%s(oar)>Read</input><br>
  if( g.okSetup ){
    @     <input type="checkbox" name="as"%s(oas)>Setup</input><br>
  }
  @     <input type="checkbox" name="aw"%s(oaw)>Write</input>
  @   </td>
  @ </tr>
  @ <tr>
  @   <td align="right">Password:</td>
  @   <td><input type="password" name="pw" value="%h(zPW)"></td>
  @ </tr>
  if( !higherUser ){
    @ <tr>
    @   <td>&nbsp</td>
    @   <td><input type="submit" name="submit" value="Apply Changes">
    @       &nbsp;&nbsp;&nbsp;
    @       <input type="submit" name="delete" value="Delete User"></td>
    @ </tr>
  }
  @ </table>
  @ <p><b>Notes:</b></p>
  @ <ol>
  if( higherUser ){
    @ <li><p>
    @ User %h(zId) has Setup privileges and you only have Admin privileges
    @ so you are not permitted to make changes to %h(zId).
    @ </p></li>
    @
  }
  @ <li><p>
  @ If the Check-out capability is specified then
  @ the password entered here will be used to regenerate the
  @ <b>CVSROOT/passwd</b> file and will thus become the CVS password
  @ as well as the password for this server.
  @ </p></li>
  @
  @ <li><p>
  @ The <b>Check-in</b> capability means that the user ID will be written
  @ into the <b>CVSROOT/writers</b> file and thus allow write access to
  @ the CVS repository.
  @ </p></li>
  @
  @ <li><p>
  @ The <b>Read</b> and <b>Write</b> privileges give the user the ability
  @ to read and write tickets.  The <b>New Tkt</b> capability means that
  @ the user is able to create new tickets.
  @ </p></li>
  @
  @ <li><p>
  @ An <b>Admin</b> user can add other users, create new ticket report
  @ formats, and change system defaults.  But only the <b>Setup</b> user
  @ is able to change the CVS repository to which this program is linked.
  @ </p></li>
  @
  if( zId==0 || strcmp(zId,"anonymous")==0 ){
    @ <li><p>
    @ No login is required for user "<b>anonymous</b>".  The capabilities
    @ of this user are available to anyone without supplying a username or
    @ password.  To disable anonymous access, make sure there is no user
    @ with an ID of <b>anonymous</b>.
    @ </p></li>
    @
    @ <li><p>
    @ The password for the "<b>anonymous</b>" user is used for anonymous
    @ CVS access.  The recommended value for the anonymous password
    @ is "anonymous".
    @ </p></li>
  }
  @ </form>
  common_footer();
}

/*
** Write the CVSROOT/passwd and CVSROOT/writer files based on the current
** state of the database.
*/
int user_cvs_config(void){
  FILE *pswd;
  FILE *wrtr;
  char **az;
  int i;
  char *zFile;
  char *zCvsRoot;
  struct passwd *pPw;
  char *zUser;

  if( geteuid()!=0 ){
    pPw = getpwuid(geteuid());
    zUser = pPw->pw_name;
  }else if( getuid()!=0 ){
    pPw = getpwuid(getuid());
    zUser = pPw->pw_name;
  }else{
    zUser = "nobody";
  }
  zCvsRoot = db_short_query("SELECT value FROM config WHERE name='cvsroot'");
  zFile = mprintf("%s/CVSROOT/passwd", zCvsRoot);
  pswd = fopen(zFile, "w");
  free(zFile);
  if( pswd==0 ) return 0;
  zFile = mprintf("%s/CVSROOT/writers", zCvsRoot);
  wrtr = fopen(zFile, "w");
  free(zFile);
  az = db_query("SELECT id, passwd, capabilities FROM user ORDER BY id");
  for(i=0; az[i]; i+=3){
    char *zCrypt;
    char zSalt[3];
    if( strchr(az[i+2],'o')==0 ) continue;
    zSalt[0] = az[i][0];
    zSalt[1] = az[i+2][0];
    zSalt[2] = 0;
    zCrypt = crypt(az[i+1],zSalt);
    fprintf(pswd, "%s:%s:%s\n", az[i], zCrypt, zUser);
    if( wrtr!=0 && strchr(az[i+2],'i')!=0 ){
      fprintf(wrtr,"%s\n", az[i]);
    }
  }
  db_query_free(az);
  fclose(pswd);
  if( wrtr ) fclose(wrtr);
  return 1;
}
